/********************************** (C) COPYRIGHT *******************************
* File Name          : pm_measure.h
* Author             : WCH
* Version            : V1.0
* Date               : 2023/06/06
* Description        :
*******************************************************************************/
#ifndef __PM_MEASURE_H
#define __PM_MEASURE_H

#ifdef __cplusplus
extern "C"
{
#endif

#include "ch32v30x.h"

/*********************************************************************************
 *                                  MACRO
 *********************************************************************************/
/* Component quick classify voltage */
#define QUICK_CLASSIFY_VOLTAGE              ((u32)100) //100mV


/*********************************************************************************
 *                                  ENUM
 *********************************************************************************/
/* ·ͨ״̬ */
typedef enum
{
    OPEN = 0,       //·
    CLOSED          //·
}PM_CircuitState;

/* ԪԽ */
typedef enum
{
    FAILURE = 0,    //ʧ
    SUCCESS         //ɹ
}PM_State;

/* ܹ */
typedef enum
{
    N_CHANNEL = 0,  //N
    P_CHANNEL       //P
}PM_TransistorChannelType;

/* Ԫ */
typedef enum
{
    COMPONENT_NONE = 0,         //Ԫ
    COMPONENT_RESISTANCE,       //
    COMPONENT_CAPACITANCE,      //
    COMPONENT_DIODE,            //
    COMPONENT_BJT,              //˫;BJT
    COMPONENT_JFET,             //JFET
    COMPONENT_EMOS,             //EMOS
    COMPONENT_DMOS,             //DMOS
}PM_ComponentType;

/* Ԫ */
typedef enum
{
    STEP_MEASURE_START = 0,         //ʼ
    STEP_MEASURE_OVER,              //
    STEP_MEASURE_RES,               //
    STEP_MEASURE_CAP,               //
    STEP_MEASURE_DIO,               //
    STEP_MEASURE_BJT,               //BJT
    STEP_CHECK_IS_CAP,              //ǷΪ
    STEP_CHECK_IS_RES,              //ǷΪ
}PM_ComponentMeasureStep;


/*********************************************************************************
 *                                  STRUCT
 *********************************************************************************/
/* GPIOѹϹʽ   Vol = Slope * Circuit + Intercept*/
typedef struct
{
    float Slope;        //б
    float Intercept;    //ؾ
}IO_VolDropParam;

/* ˿ڶ */
typedef struct
{
    GPIO_TypeDef *GPIOx;
    u16 GPIO_Pin;
    IO_VolDropParam PMOS;   //GPIO PMOSѹʽ
    IO_VolDropParam NMOS;   //GPIO NMOSѹʽ
}PM_PortDef;

/* ˿(Խ) */
typedef struct
{
    PM_PortDef PortWithNone;    //δĶ˿ڶ
    PM_PortDef PortWith680;     //680˿ڶ
    PM_PortDef PortWith470K;    //470K˿ڶ
    u8 ADC_Channel;             //ʹõADCͨ
    u8 Probe_Num;               //ʺ
}PM_MeasurePortGroup;

/* ٷ */
typedef struct
{
    u8 Path_1to2;      //12
    u8 Path_2to1;      //21
    u8 Path_2to3;      //23
    u8 Path_3to2;      //32
    u8 Path_1to3;      //13
    u8 Path_3to1;      //31
    u8 Feature[3];
}PM_ComponentFeature;

/*  */
typedef struct
{
    u8 Front;
    u8 Rear;
    double ResVal; //ֵ,
}PM_ResParam;

/* ݲ */
typedef struct
{
    u8 Front;
    u8 Rear;
    double CapVal; //ֵ,nF
}PM_CapParam;

/* ܲ */
typedef struct
{
    u8 Positive; //
    u8 Negative; //
    u32 Uon;     //ͨѹ
    double Cj;    //
}PM_DiodeParam;

/* BJT */
typedef struct
{
    u8 Channel; //
    u8 C;       //缫ͨ
    u8 B;       //ͨ
    u8 E;       //伫ͨ
    u16 hFE;
    s32 Ube;
}PM_BJTParam;

/* JFET */
typedef struct
{
    u8 Channel; //
    u8 D;       //©ͨ
    u8 G;       //դͨ
    u8 S;       //Դͨ
    u32 Uoff;   //жϵѹ
    double Idss; //©
}PM_JFETParam;

/* EMOS */
typedef struct
{
    u8 Channel; //
    u8 D;       //©ͨ
    u8 G;       //դͨ
    u8 S;       //Դͨ
    u32 Uds;    //©Դ֮ѹ
    u32 Ud;     //ڲܵĵͨѹ
}PM_EMOSFETParam;

/* DMOS */
typedef struct
{
    u8 Channel; //
    u8 D;       //©ͨ
    u8 G;       //դͨ
    u8 S;       //Դͨ
    u32 Uoff;   //жϵѹ
    double Idss; //©
    u32 Ud;     //ڲܵĵͨѹ
}PM_DMOSFETParam;

/* Ԫ */
typedef union
{
    PM_ResParam         ResParam;       //
    PM_CapParam         CapParam;       //ݲ
    PM_DiodeParam       DiodeParam;     //ܲ
    PM_BJTParam         BJTParam;       //BJT
    PM_JFETParam        JFETParam;      //JFET
    PM_EMOSFETParam     EMOSFETParam;   //EMOS
    PM_DMOSFETParam     DMOSFETParam;   //DMOS
}PM_ComponentParameter;

/* Ԫ */
typedef struct
{
    PM_ComponentType        ComponentType;  //Ԫ
    PM_ComponentParameter   ComponentParam; //Ԫ
}PM_ComponentMeasureResult;


/*******************************************************************************
 *                           GLOBAL VARIABLES
 *******************************************************************************/
/* Ԫʼ־ */
extern volatile u8 MeasureStartFlg;

/* Ԫ */
extern volatile PM_ComponentMeasureStep MeasureStep;

/* ʱ */
extern volatile u16 TimerOverflowNum;

/* ʲ */
extern const PM_MeasurePortGroup PortGroup_1;
extern const PM_MeasurePortGroup PortGroup_2;
extern const PM_MeasurePortGroup PortGroup_3;

/* Ԫ */
extern PM_ComponentMeasureResult ComponentMeasureResult;

/* ٷ */
extern PM_ComponentFeature ComponentFeature;

/* ĵ裨Ҫʵ£ */
extern float PM_Probe1Res; //1ĵ
extern float PM_Probe2Res; //2ĵ
extern float PM_Probe3Res; //3ĵ


/********************************************************************************
 *                                   FUNCTIONS
 ********************************************************************************/
u8 PM_ComponentQuickClassify(void);
void PM_ComponentDischarge(void);
u8 PM_ComponentFeatureCompare(u8 *pArr1, u8 *pArr2, u8 len);


/********************************************************************************
*********************************************************************************/

#ifdef __cplusplus
}
#endif

#endif
