/********************************** (C) COPYRIGHT *******************************
* File Name          : main.c
* Author             : WCH
* Version            : V1.0.0
* Date               : 2021/06/06
* Description        : Main program body.
*********************************************************************************
* Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
* Attention: This software (modified or not) and binary are used for 
* microcontroller manufactured by Nanjing Qinheng Microelectronics.
*******************************************************************************/

#include "debug.h"
#include "pm_measure.h"
#include "pm_adc.h"
#include "pm_key.h"
#include "pm_timer.h"
#include "pm_resistance.h"
#include "pm_lcd.h"
#include "pm_capacitor.h"
#include "pm_diode.h"
#include "pm_bjt.h"
#include "pm_display.h"
#include "string.h"
#include "pm_ch9141.h"


#define CONFIG_MAIN_DEBUG
#ifdef CONFIG_MAIN_DEBUG
#define LOG_INFO(format, ...)       printf("<%s:%d> "format, __FILE__, __LINE__, ##__VA_ARGS__)
//#define LOG_INFO(...)       printf(__VA_ARGS__)
#else
#define LOG_INFO(format, ...)
#endif

/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *
 * @return  none
 */
int main(void)
{
	NVIC_PriorityGroupConfig(NVIC_PriorityGroup_2);
	SystemCoreClockUpdate();
	Delay_Init();
	USART_Printf_Init(115200);	
	LOG_INFO("%s\r\n", __TIME__);
	LOG_INFO("SystemClk:%d\r\n", SystemCoreClock);
	LOG_INFO("ChipID:%08x\r\n", DBGMCU_GetCHIPID());

    PM_LCDInit();       //LCDʼ
    PM_CH9141Init();    //CH9141ʼ
	PM_ADCInit();       //ADCʼ
    PM_KeyInit();       //ʼ
    PM_TimerInit(TIMER_COUNT_PERIOD, TIMER_PRESCALER); //ʱʼ
    PM_DisplayReady();  //豸ʼʾ

	while(1)
    {
	    if(MeasureStartFlg == 1)
	    {
	        PM_DisplayTesting();
            MeasureStartFlg = 0;
            MeasureStep = STEP_MEASURE_START;
            ComponentMeasureResult.ComponentType = COMPONENT_NONE;

	        while(MeasureStartFlg == 0) //;£˳ѭ
	        {
                switch(MeasureStep)
                {
                case STEP_MEASURE_START:
                    LOG_INFO("STEP_MEASURE_START\r\n");
                    MeasureStep = PM_ComponentQuickClassify();
                    break;

                case STEP_CHECK_IS_CAP:
                    LOG_INFO("STEP_CHECK_IS_CAP\r\n");
                    if (PM_CheckIsCapacitor() == SUCCESS)
                    {
                        MeasureStep = STEP_MEASURE_CAP;
                    }
                    else
                    {
                        if(ComponentFeature.Feature[0] == 2)
                        {
                            MeasureStep = STEP_MEASURE_RES; //
                            ComponentMeasureResult.ComponentType = COMPONENT_RESISTANCE;
                            ComponentMeasureResult.ComponentParam.ResParam.Front = PortGroup_1.Probe_Num; //޼ԣĬԱСķǰ
                            ComponentMeasureResult.ComponentParam.ResParam.Rear = PortGroup_2.Probe_Num;
                        }
                        else if(ComponentFeature.Feature[1] == 2)
                        {
                            MeasureStep = STEP_MEASURE_RES; //
                            ComponentMeasureResult.ComponentType = COMPONENT_RESISTANCE;
                            ComponentMeasureResult.ComponentParam.ResParam.Front = PortGroup_2.Probe_Num; //޼ԣĬԱСķǰ
                            ComponentMeasureResult.ComponentParam.ResParam.Rear = PortGroup_3.Probe_Num;
                        }
                        else if(ComponentFeature.Feature[2] == 2)
                        {
                            MeasureStep = STEP_MEASURE_RES; //
                            ComponentMeasureResult.ComponentType = COMPONENT_RESISTANCE;
                            ComponentMeasureResult.ComponentParam.ResParam.Front = PortGroup_1.Probe_Num; //޼ԣĬԱСķǰ
                            ComponentMeasureResult.ComponentParam.ResParam.Rear = PortGroup_3.Probe_Num;
                        }
                        else
                        {
                            MeasureStep = STEP_CHECK_IS_RES; //жǷΪ
                        }
                    }
                    break;

                case STEP_MEASURE_CAP:
                    LOG_INFO("STEP_MEASURE_CAP\r\n");
                    if (PM_MeasureCapacitor() != SUCCESS)
                    {
                        ComponentMeasureResult.ComponentType = COMPONENT_NONE;
                    }
                    MeasureStep = STEP_MEASURE_OVER;
                    break;

                case STEP_CHECK_IS_RES:
                    LOG_INFO("STEP_CHECK_IS_RES\r\n");
                    if (PM_CheckIsResistance() == SUCCESS)
                    {
                        MeasureStep = STEP_MEASURE_RES;
                    }
                    else
                    {
                        MeasureStep = STEP_MEASURE_OVER;
                    }
                    break;

                case STEP_MEASURE_RES:
                    LOG_INFO("STEP_MEASURE_RES\r\n");
                    if (PM_MeasureResistance() != SUCCESS)
                    {
                        ComponentMeasureResult.ComponentType = COMPONENT_NONE;
                    }
                    MeasureStep = STEP_MEASURE_OVER;
                    break;

                case STEP_MEASURE_DIO:
                    LOG_INFO("STEP_MEASURE_DIO\r\n");
                    if (PM_MeasureDiode()!= SUCCESS)
                    {
                        ComponentMeasureResult.ComponentType = COMPONENT_NONE;
                    }
                    MeasureStep = STEP_MEASURE_OVER;
                    break;

                case STEP_MEASURE_BJT:
                    LOG_INFO("STEP_MEASURE_BJT\r\n");
                    if (PM_MeasureBJT()!= SUCCESS)
                    {
                        ComponentMeasureResult.ComponentType = COMPONENT_NONE;
                    }
                    MeasureStep = STEP_MEASURE_OVER;
                    break;

                default:
                    LOG_INFO("Error\r\n");
                    ComponentMeasureResult.ComponentType = COMPONENT_NONE;
                    MeasureStep = STEP_MEASURE_OVER;
                    break;
                }

                if(MeasureStep == STEP_MEASURE_OVER) //˳ѭ
                {
                    LOG_INFO("STEP_MEASURE_OVER\r\n");
                    switch (ComponentMeasureResult.ComponentType)
                    {
                    case COMPONENT_RESISTANCE:
                        PM_DisplayResistance(&ComponentMeasureResult);
                        break;
                    case COMPONENT_CAPACITANCE:
                        PM_DisplayCapacitor(&ComponentMeasureResult);
                        break;
                    case COMPONENT_DIODE:
                        PM_DisplayDiode(&ComponentMeasureResult);
                        break;
                    case COMPONENT_BJT:
                        PM_DisplayBJT(&ComponentMeasureResult);
                        break;
                    default:
                        PM_DisplayError();
                        break;
                    }
                    break;
                }
	        }
	    }
	}
}
